/*
 *  The Mana Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *
 *  This file is part of The Mana Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ITEM_AMOUNT_WINDOW_H
#define ITEM_AMOUNT_WINDOW_H

#include "gui/widgets/window.h"

#include <guichan/keylistener.hpp>
#include <guichan/actionlistener.hpp>

#ifdef __GNUC__
#define _UNUSED_  __attribute__ ((unused))
#else
#define _UNUSED_
#endif

class Icon;
class IntTextField;
class Item;
class ItemsModal;
class ItemPopup;
class Label;

/**
 * Window used for selecting the amount of items to drop, trade or split.
 *
 * \ingroup Interface
 */
class ItemAmountWindow : public Window,
                         public gcn::ActionListener,
                         public gcn::KeyListener
{
    public:
        enum Usage
        {
            TradeAdd = 0,
            ItemDrop,
            StoreAdd,
            StoreRemove,
            ItemSplit,
            ShopBuyAdd,
            ShopSellAdd
        };

        /**
         * Called when receiving actions from widget.
         */
        void action(const gcn::ActionEvent &event);

        /**
         * Sets default amount value.
         */
        void resetAmount();

        // MouseListener
        void mouseMoved(gcn::MouseEvent &event);
        void mouseExited(gcn::MouseEvent &event);

        /**
         * Schedules the Item Amount window for deletion.
         */
        void close();

        void keyReleased(gcn::KeyEvent &keyEvent);

        /**
         * Creates the dialog, or bypass it if there aren't enough items.
         */
        static void showWindow(Usage usage, Window *parent, Item *item,
                               int maxRange = 0);

        ~ItemAmountWindow();

    private:
        static void finish(Item *item, int amount, int price, Usage usage);

        ItemAmountWindow(Usage usage, Window *parent, Item *item,
                         int maxRange = 0);

        IntTextField *mItemAmountTextField;   /**< Item amount caption. */
        IntTextField *mItemPriceTextField;   /**< Item price caption. */
        Label *mGPLabel;
        Item *mItem;
        Icon *mItemIcon;

        int mMax;
        Usage mUsage;
        ItemPopup *mItemPopup;

        /**
         * Item Amount buttons.
         */
        gcn::Slider *mItemAmountSlide;

        gcn::Slider *mItemPriceSlide;

        gcn::DropDown *mItemDropDown;

        ItemsModal *mItemsModal;

        bool mEnabledKeyboard;
        int mPrice;
};

#endif // ITEM_AMOUNT_WINDOW_H
