/*
 *  The Mana World
 *  Copyright (C) 2004  The Mana World Development Team
 *
 *  This file is part of The Mana World.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "homelocation.h"

#include "animatedsprite.h"
#include "configuration.h"
#include "effectmanager.h"
#include "game.h"
#include "graphics.h"
#include "localplayer.h"
#include "log.h"
#include "map.h"
#include "particle.h"
#include "simpleanimation.h"
#include "sound.h"
#include "text.h"
#include "statuseffect.h"
#include "monster.h"

#include "gui/speechbubble.h"

#include "resources/colordb.h"
#include "resources/emotedb.h"
#include "resources/image.h"
#include "resources/itemdb.h"
#include "resources/iteminfo.h"
#include "resources/resourcemanager.h"

#include "gui/gui.h"
#include "gui/palette.h"
#include "gui/speechbubble.h"

#include "utils/dtor.h"
#include "utils/gettext.h"
#include "utils/stringutils.h"
#include "utils/xml.h"

#include <cassert>
#include <cmath>

HomeLocation::HomeLocation():
    mPx(-1), mPy(-1)
{
}

HomeLocation::~HomeLocation()
{
}

void HomeLocation::draw(Graphics *graphics, int offsetX, int offsetY) const
{
    const int px = mPx + offsetX - 16;
    const int py = mPy + offsetY - 32;

    graphics->setColor(gcn::Color(200, 0, 0, 64));
    graphics->fillRectangle(gcn::Rectangle(
                                            px, py,
                                            32, 32));
}

int HomeLocation::getWidth() const
{
    return mPx > -1 ? 16 : 0;
}

int HomeLocation::getHeight() const
{
    return mPx > -1 ? 16 : 0;
}

void HomeLocation::setPosition(const Vector &pos)
{
    mPx = (int) pos.x;
    mPy = (int) pos.y;
}
